"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Text_1 = tslib_1.__importDefault(require("./Text"));
var SPCoreLibraryStrings_resx_1 = tslib_1.__importDefault(require("./SPCoreLibraryStrings.resx"));
/**
 * Performs common validation tests for properties and function parameters.
 *
 * @remarks
 *
 * This class implements provides a standard way to validate properties and function parameters.
 * Unlike debug assertions, Validate checks are always performed and will always throw an error,
 * even in a production release.  As such, be careful not to overuse these checks in a way
 * that might impact performance.
 *
 * @public
 */
var Validate = /** @class */ (function () {
    function Validate() {
    }
    /**
     * Throws an exception if the specified value is not true.
     * @param value - the value to check
     * @param variableName - the program variable name, which will be mentioned in the error message
     */
    // eslint-disable-next-line @rushstack/no-new-null
    Validate.isTrue = function (value, variableName) {
        if (!value) {
            throw new Error(Text_1.default.format(SPCoreLibraryStrings_resx_1.default.isFalseValidateError, variableName));
        }
    };
    /**
     * Throws an exception if the specified value is null or undefined.
     * @param value - the value to check
     * @param variableName - the program variable name, which will be mentioned in the error message
     */
    Validate.isNotNullOrUndefined = function (value, variableName) {
        if (value === null) {
            throw new Error(Text_1.default.format(SPCoreLibraryStrings_resx_1.default.isNullValidateError, variableName));
        }
        if (value === undefined) {
            throw new Error(Text_1.default.format(SPCoreLibraryStrings_resx_1.default.isUndefinedValidateError, variableName));
        }
    };
    /**
     * Throws an exception if the specified string is null, undefined, or an empty string.
     * @param value - the value to check
     * @param variableName - the program variable name, which will be mentioned in the error message
     */
    // eslint-disable-next-line @rushstack/no-new-null
    Validate.isNonemptyString = function (value, variableName) {
        Validate.isNotNullOrUndefined(value, variableName);
        if (!value || (value && value.length === 0)) {
            throw new Error(Text_1.default.format(SPCoreLibraryStrings_resx_1.default.isEmptyStringValidateError, variableName));
        }
    };
    /**
     * Throws an exception if the specified object has been disposed.
     * @param value - the value to check
     * @param className - the class name, which will be mentioned in the error message
     */
    Validate.isNotDisposed = function (value, className) {
        if (value.isDisposed) {
            throw new Error(Text_1.default.format(SPCoreLibraryStrings_resx_1.default.isDisposedValidateError, className));
        }
    };
    return Validate;
}());
exports.default = Validate;
/*
  Internal notes:
  - We will improve the exception and localization of error messages in VSO #163788
  - Do not add validation algorithms (e.g. isZipCode()) to this file; it should only
    provide basic checks similar to Assert
*/
//# sourceMappingURL=Validate.js.map