"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Text_1 = tslib_1.__importDefault(require("./Text"));
var Validate_1 = tslib_1.__importDefault(require("./Validate"));
var SPCoreLibraryStrings_resx_1 = tslib_1.__importDefault(require("./SPCoreLibraryStrings.resx"));
/**
 * Represents a version number with two, three, or four parts.
 *
 * @remarks
 * This class represents versions that follow the string format of `MAJOR.MINOR[.PATCH[.REVISION]]`
 * where the MAJOR, MINOR, PATCH and REVISION parts are integers. PATCH and REVISION are optional.
 * Leading zero digits are allowed, but they are discarded.  Missing parts are treated
 * as zeroes for comparisons.
 *
 * Examples: `1.0`, `1.0.0`, `1.0.0.0`, `1.01`, `01.02.03`, `001.002.003.004`
 *
 * @public
 */
var Version = /** @class */ (function () {
    /**
     * WARNING: Use Version.tryParse instead of the constructor. The constructor does not do any validation.
     *
     * @param major - the major version
     * @param minor - the minor version
     * @param patch - the patch number
     * @param revision - the revision number
     */
    function Version(major, minor, patch, revision) {
        this._major = major;
        this._minor = minor;
        this._patch = patch;
        this._revision = revision;
    }
    /**
     * Test whether a string is a valid version specifier.
     *
     * @param versionString - The version string
     * @returns true if the versionString is a valid version specifier
     */
    // eslint-disable-next-line @rushstack/no-new-null
    Version.isValid = function (versionString) {
        return Version.tryParse(versionString) !== undefined;
    };
    /**
     * Constructs a new Version instance using the version string.  An exception is thrown
     * if the string cannot be parsed.
     *
     * @param versionString - A version string
     * @returns a new Version object
     */
    // eslint-disable-next-line @rushstack/no-new-null
    Version.parse = function (versionString) {
        var version = Version.tryParse(versionString);
        if (version) {
            return version;
        }
        throw new Error(Text_1.default.format(SPCoreLibraryStrings_resx_1.default.invalidVersionStringError, versionString));
    };
    /**
     * Attempts to parse the input string to construct a new Version object.
     * If the string cannot be parsed, then undefined is returned.
     *
     * @param versionString - A version string
     * @returns The Version object, or undefined if the string could not be parsed.
     */
    // eslint-disable-next-line @rushstack/no-new-null
    Version.tryParse = function (versionString) {
        if (versionString) {
            var versionRegex = /^([0-9]+)\.([0-9]+)(?:\.([0-9]+)(?:\.([0-9]+))?)?$/g;
            var regexExecRes = versionRegex.exec(versionString);
            if (regexExecRes) {
                return new Version(parseInt(regexExecRes[1], 10), parseInt(regexExecRes[2], 10), regexExecRes[3] !== undefined ? parseInt(regexExecRes[3], 10) : undefined, regexExecRes[4] !== undefined ? parseInt(regexExecRes[4], 10) : undefined);
            }
        }
        return undefined;
    };
    /**
     * Compares two Version objects to determine which version is newer.
     *
     * @param v1 - The first version class for comparison
     * @param v2 - The second version class for comparison
     * @returns  -1 if the first input is less than the second input;
     *            0 if the first input is equal to the second input;
     *            1 if the first input is greater than the second input.
     */
    Version.compare = function (v1, v2) {
        Validate_1.default.isNotNullOrUndefined(v1, 'v1');
        Validate_1.default.isNotNullOrUndefined(v2, 'v2');
        if (v1.major !== v2.major) {
            return v1.major > v2.major ? 1 : -1;
        }
        if (v1.minor !== v2.minor) {
            return v1.minor > v2.minor ? 1 : -1;
        }
        if ((v1.patch || 0) !== (v2.patch || 0)) {
            return (v1.patch || 0) > (v2.patch || 0) ? 1 : -1;
        }
        if ((v1.revision || 0) !== (v2.revision || 0)) {
            return (v1.revision || 0) > (v2.revision || 0) ? 1 : -1;
        }
        return 0;
    };
    /**
     * Get the version from given manifest.
     * @param manifest - The manifest where to get version.
     * @returns - The version of the manifest or undefined if version cannot be obtained.
     * @internal
     */
    Version._tryParseSPFxVersion = function (manifest) {
        try {
            var resource = manifest.loaderConfig.scriptResources['@microsoft/sp-core-library'];
            return Version.parse(resource.version);
        }
        catch (err) {
            return undefined;
        }
    };
    Object.defineProperty(Version.prototype, "major", {
        /**
         * Returns the first component of the version string.
         *
         * @remarks
         * Typically a change in the major version number indicates a compatibility
         * break with previous versions.
         */
        get: function () {
            return this._major;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Version.prototype, "minor", {
        /**
         * Returns the second component of the version string.
         *
         * @remarks
         * Typically a change in the minor version number indicates that new features
         * were added, while remaining backwards compatible with previous releases.
         */
        get: function () {
            return this._minor;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Version.prototype, "patch", {
        /**
         * The third number in the version string, or undefined if unspecified.
         *
         * @remarks
         * Typically a change in the patch version number indicates a small fix that
         * does not affect the compatibility contract for the library.  For a .NET
         * System.Version object, this is referred to as the "build" number.
         */
        get: function () {
            return this._patch;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Version.prototype, "revision", {
        /**
         * The fourth number in the version string, or undefined if unspecified.
         *
         * @remarks
         * This number is not part of the Semantic Versioning (SemVer) standard used
         * in JavaScript, but it is used by .NET version numbers.
         */
        get: function () {
            return this._revision;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Tests whether this version is less than (i.e. older than) the input parameter.
     *
     * @remarks
     *
     * Examples:
     *
     * ```
     * 0.9.9 lessThan 1.0.0 -> true;
     * 2.0   lessThan 2.0.0 -> false;
     * 3.0   lessThan 3.0.1 -> true;
     * 04.01 lessThan 4.1   -> false
     * ```
     *
     * @param compareWith - The version to compare with
     * @returns A boolean indicating if this version is less than the input parameter
     */
    Version.prototype.lessThan = function (compareWith) {
        return Version.compare(this, compareWith) < 0;
    };
    /**
     * Tests whether this version is greater than (i.e. newer than) the input parameter.
     *
     * @remarks
     *
     * Examples:
     *
     * ```
     * 1.0.0 greaterThan 0.0.9 -> true;
     * 2.0   greaterThan 2.0.0 -> false;
     * 3.0.1 greaterThan 3.0   -> true
     * ```
     *
     * @param compareWith - The version to compare with
     * @returns A boolean indicating if this version is greater than the input parameter
     */
    Version.prototype.greaterThan = function (compareWith) {
        return Version.compare(this, compareWith) > 0;
    };
    /**
     * Tests whether this version is equal to the input parameter.
     *
     * @remarks
     *
     * Examples:
     *
     * ```
     * 1.0.0 equals 1.0.0 -> true;
     * 2.0.1 equals 2.0.0 -> false;
     * 3.0   equals 3.0.0 -> true;
     * 04.01 equals 4.1   -> true
     * ```
     *
     * @param compareWith - The version to compare with
     * @returns A boolean indicating if this version is equal to the input parameter
     */
    Version.prototype.equals = function (compareWith) {
        return Version.compare(this, compareWith) === 0;
    };
    /**
     * Tests whether this version satisfies the compatibility requirements of the input version,
     * i.e. is backwards compatible.
     *
     * @remarks
     * In order to satisfy the compatibility requirements, this object must have the same
     * major version number as the input parameter, and it must NOT be older than the
     * input parameter.
     *
     * Examples:
     *
     * ```
     * 1.0.0 satisfies 1.0.0 -> true;
     * 1.1.0 satisfies 1.0.0 -> true;
     * 2.0.0 satisfies 1.0.0 -> false;
     * 1.0.0 satisfies 1.1.0 -> false
     * ```
     *
     * @param compareWith - The version to compare with
     * @returns A boolean indicating if this version is compatible with the input parameter
     */
    Version.prototype.satisfies = function (compareWith) {
        Validate_1.default.isNotNullOrUndefined(compareWith, 'compareWith');
        return this.major === compareWith.major && !this.lessThan(compareWith);
    };
    /**
     * Returns a string representation of the version.
     *
     * @remarks
     * The value is normalized and may be different from the original string (e.g. leading zeroes
     * may be removed).  However, the number of version parts will be unchanged.
     */
    Version.prototype.toString = function () {
        var str = "".concat(this.major, ".").concat(this.minor);
        if (this.patch !== undefined) {
            str += ".".concat(this.patch);
            if (this.revision !== undefined) {
                str += ".".concat(this.revision);
            }
        }
        return str;
    };
    return Version;
}());
exports.default = Version;
//# sourceMappingURL=Version.js.map