"use strict";
// Copyright (c) Microsoft. All rights reserved.
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Validate_1 = tslib_1.__importDefault(require("../Validate"));
var URL_PROTOCOL_REGEX = /^\w+\:\/\//;
// data:[<mediatype>][;base64],<data>
var BASE64_IMAGE_REGEX = /^data:(\w+\/\w+(\+\w+)?)?(;charset=[\w-]+)?(;base64)?,.*/i;
/**
 * Common helper functions for working with URLs.  These utilities are intended to be simple,
 * small, and very broadly applicable.
 *
 * @alpha
 */
var UrlUtilities = /** @class */ (function () {
    function UrlUtilities() {
    }
    /**
     * If the provided URL is relative, resolve it under the provided base URL. If the provided URL is absolute,
     * return it.
     *
     * @remarks
     * Examples:
     * ```
     *   resolve('/foo/bar.js', 'http://cdn.com/path/')               ---> 'http://cdn.com/path/foo/bar.js'
     *   resolve('http://cdn.com/foo/bar.js', 'http://cdn.com/path/') ---> 'http://cdn.com/foo/bar.js'
     *   resolve('/foo/bar.js', 'path/')                              ---> 'path/foo/bar.js'
     * ```
     * @param url - the URL to be resolved
     * @param baseUrl - the base URL to use if the URL is relative
     */
    UrlUtilities.resolve = function (url, baseUrl) {
        Validate_1.default.isNonemptyString(url, 'url');
        Validate_1.default.isNonemptyString(baseUrl, 'baseUrl');
        // If the icon URL doesn't start with a protocol handler (i.e. - "http://") assume it's a relative URL and
        //  append it to the loaderConfig's internalModuleBaseUrl
        var isRelativeUrl = UrlUtilities.isRelativeUrl(url);
        if (isRelativeUrl) {
            return "".concat(UrlUtilities.removeEndSlash(baseUrl), "/").concat(UrlUtilities.removeLeadingSlash(url));
        }
        else {
            return url;
        }
    };
    /**
     * Removes any slash characters from the end of the URL.
     *
     * @remarks
     * This function assumes that the input is already a valid absolute or server-relative URL.
     *
     * Examples:
     * ```
     *   removeEndSlash('http://example.com/') ---> 'http://example.com'
     *   removeEndSlash('/example')            ---> '/example'
     *   removeEndSlash('/')                   ---> ''
     * ```
     * @param url - the URL to be normalized
     */
    UrlUtilities.removeEndSlash = function (url) {
        return url.replace(/\/+$/, '');
    };
    /**
     * Removes any slash characters from the beginning of the URL.
     *
     * @remarks
     * This function assumes that the input is already a valid absolute or server-relative URL.
     *
     * Examples:
     * ```
     *   removeLeadingSlash('/example/path.js') ---> 'example/path.js'
     *   removeLeadingSlash('example/')         ---> 'example/'
     *   removeLeadingSlash('/')                ---> ''
     * ```
     * @param url - the URL to be normalized
     */
    UrlUtilities.removeLeadingSlash = function (url) {
        return url.replace(/^\/+/, '');
    };
    /**
     * Converts a variable to an OData string literal suitable for usage in a REST URL.
     *
     * @remarks
     * The returned string will be enclosed in single quotes, and any single quotes
     * will be escaped.
     *
     * Example usage:
     *
     * ```
     * const url = "/_api/web/GetFolderByServerRelativeUrl("
     *       + UrlUtilities.convertToODataStringLiteral("/SitePages/Alice's%20Page")
     *       + ")/Files";
     *
     *  // Produces this URL:
     *  // "/_api/web/GetFolderByServerRelativeUrl('/SitePages/Alice''s%20Page')/Files"
     * ```
     *
     * @privateRemarks
     * Standard reference:
     * https://tools.oasis-open.org/version-control/
     *   browse/wsvn/odata/trunk/spec/ABNF/odata-abnf-construction-rules.txt
     * SQUOTE-in-string = SQUOTE SQUOTE ; two consecutive single quotes represent
     *   one within a string literal
     */
    UrlUtilities.convertToODataStringLiteral = function (value) {
        Validate_1.default.isNotNullOrUndefined(value, 'value');
        value = value.replace(/'/g, "''");
        value = "'".concat(value, "'");
        return value;
    };
    /**
     * Detects if a string represents a correctly formatted data URL.
     *
     * @remarks
     *
     * Examples:
     * ```
     *   isDataUrl('data:text/plain;base64,SGVsbG8sIFdvcmxkIQ%3D%3D') ---> true
     *   isDataUrl('data:,Hello%2C%20World!')                         ---> true
     *   isDataUrl('data:image/png;base64,iVBORw0KGgoAA')             ---> true
     *   isDataUrl('http://contoso.com')                              ---> false
     *   isDataUrl('/foo/bar')                                        ---> false
     * ```
     *
     * Information on data URLs can be found at https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/Data_URIs
     */
    UrlUtilities.isDataUrl = function (url) {
        Validate_1.default.isNotNullOrUndefined(url, 'url');
        return !!url.match(BASE64_IMAGE_REGEX);
    };
    /**
     * Detects if a url is relative
     *
     * @remarks
     * This function assumes that the input is already a valid absolute or server-relative URL.
     *
     * Examples:
     * ```
     *   isRelativeUrl('/example/path.aspx')  ---> true
     *   isRelativeUrl('http://contoso.com')  ---> false
     * ```
     */
    UrlUtilities.isRelativeUrl = function (url) {
        return !url.match(URL_PROTOCOL_REGEX);
    };
    /**
     * Detects if a url is absolute url
     *
     * Examples:
     * ```
     *   isAbsoluteUrl('http://example.com')  ---> true
     *   isAbsoluteUrl('/test/123')  ---> false
     * ```
     */
    UrlUtilities.isAbsoluteUrl = function (url) {
        return /^([a-z][a-z\d\-]*:)?\/\//i.test(url);
    };
    /**
     * Creates an absolute URL from a relative URL and a base absolute URL.
     */
    UrlUtilities.absolutizeUrl = function (url, baseAbsoluteUrl) {
        if (UrlUtilities.isAbsoluteUrl(url)) {
            return url;
        }
        return new URL(url, baseAbsoluteUrl).href;
    };
    /**
     * Creates a relative URL from an absolute URL.
     */
    UrlUtilities.relativizeUrl = function (url, removeQueryString) {
        if (UrlUtilities.isRelativeUrl(url)) {
            return url;
        }
        var urlObj = new URL(url);
        return urlObj.pathname + (removeQueryString ? '' : urlObj.search + urlObj.hash);
    };
    /**
     * Relativize URL to the web URL.
     */
    UrlUtilities.webRelativize2 = function (path, webAbsoluteUrl) {
        var abs = this.absolutizeUrl(path, webAbsoluteUrl);
        if (abs.toLocaleLowerCase().indexOf(webAbsoluteUrl.toLocaleLowerCase()) === 0) {
            var normalized = abs.substring(webAbsoluteUrl.length);
            return '.' + (normalized.charAt(0) === '/' ? normalized : '/' + normalized);
        }
        return path;
    };
    return UrlUtilities;
}());
exports.default = UrlUtilities;
//# sourceMappingURL=UrlUtilities.js.map